<?php

namespace Drupal\face_login_gauth\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\face_login_gauth\FaceAuthSecretTrait;
use Drupal\face_login_gauth\Response\QRImageResponse;
use Drupal\user\UserDataInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use PragmaRX\Google2FA\Google2FA;
use Drupal\Core\Session\AccountProxy;
use BaconQrCode\Renderer\ImageRenderer;
use BaconQrCode\Renderer\Image\ImagickImageBackEnd;
use BaconQrCode\Renderer\RendererStyle\RendererStyle;
use BaconQrCode\Writer;

/**
 * Class FaceLoginQrCodeGeneratorController.
 */
class FaceLoginQrCodeGeneratorController extends ControllerBase {

  use FaceAuthSecretTrait;

  // Qr code image size.
  CONST SIZE = 400;
  // Secret key length.
  CONST SECRET = 16;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * use Drupal\user\UserDataInterface; definition.
   *
   * @var \use Drupal\user\UserDataInterface;
   */
  protected $userData;
  /**
   * Constructs a new FaceLoginQrCodeGeneratorController object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, AccountProxy $currentUser, UserDataInterface $userData) {
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->currentUser = $currentUser;
    $this->userData =  $userData;
   }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('request_stack'),
      $container->get('current_user'),
      $container->get('user.data')
    );
  }

  /**
   * Generate Qr code.
   *
   * @return QRImageResponse
   * @throws \PragmaRX\Google2FA\Exceptions\IncompatibleWithGoogleAuthenticatorException
   * @throws \PragmaRX\Google2FA\Exceptions\InvalidCharactersException
   */
  public function get() {
    try {
      $google2fa = new Google2FA();
      $secret = $google2fa->generateSecretKey(self::SECRET);
      $this->deleteUserData('face_login_gauth', 'secret', $this->currentUser->id(), $this->userData);
      $this->setUserData('face_login_gauth', ['secret' => $secret], $this->currentUser->id(), $this->userData);
      if ($this->currentUser->isAuthenticated()) {
        $inlineUrl = $google2fa->getQRCodeUrl(
          $this->requestStack->getCurrentRequest()->getHttpHost(), $this->currentUser->getEmail(), $secret
        );
        $writer = $this->getWriter();
        return new QRImageResponse($writer->writeString($inlineUrl));
      }
    }
    catch(\Exception $exception) {
      // To DO.
    }
  }

  /**
   * Get Writer Object.
   *
   * @return Writer
   */
  private function getWriter() {
    return new Writer($this->getImageRenderer());
  }

  /**
   * Get ImageRenderer Object.
   *
   * @return ImageRenderer
   */
  private function getImageRenderer() {
    return new ImageRenderer(
      $this->getRendererStyle(),
      $this->getImageMagicBackend()
    );
  }

  /**
   * Get ImageMagicBackend Object.
   *
   * @return ImagickImageBackEnd
   */
  private function getImageMagicBackend() {
    return new ImagickImageBackEnd();
  }

  /**
   * Get Renderer Object.
   *
   * @return RendererStyle
   */
  private function getRendererStyle() {
    return new RendererStyle(self::SIZE);
  }

}
